// Project imports:
import 'package:remever/common/widgets/typography.dart';

/// -- Regular --

class Regular12px extends TypographyTypeRegular {
  @override
  double get size => 12;
}

class Regular14px extends TypographyTypeRegular {
  @override
  double get size => 14;
}

class Regular16px extends TypographyTypeRegular {
  @override
  double get size => 16;
}

class Regular17px extends TypographyTypeRegular {
  @override
  double get size => 17;
}

// -- Medium --

class Medium12px extends TypographyTypeMedium {
  @override
  double get size => 12;
}

class Medium13px extends TypographyTypeMedium {
  @override
  double get size => 13;
}

class Medium14px extends TypographyTypeMedium {
  @override
  double get size => 14;
}

class Medium16px extends TypographyTypeMedium {
  @override
  double get size => 16;
}

// -- SemiBold --

class SemiBold10px extends TypographyTypeSemiBold {
  @override
  double get size => 10;
}

class SemiBold11px extends TypographyTypeSemiBold {
  @override
  double get size => 11;
}

class SemiBold12px extends TypographyTypeSemiBold {
  @override
  double get size => 12;
}

class SemiBold13px extends TypographyTypeSemiBold {
  @override
  double get size => 13;
}

class SemiBold14px extends TypographyTypeSemiBold {
  @override
  double get size => 14;
}

class SemiBold20px extends TypographyTypeSemiBold {
  @override
  double get size => 20;
}

class SemiBold24px extends TypographyTypeSemiBold {
  @override
  double get size => 24;
}

class SemiBold28px extends TypographyTypeSemiBold {
  @override
  double get size => 28;
}
// -- Bold --

class Bold10px extends TypographyTypeBold {
  @override
  double get size => 10;
}

class Bold12px extends TypographyTypeBold {
  @override
  double get size => 12;
}

class Bold14px extends TypographyTypeBold {
  @override
  double get size => 14;
}

class Bold34px extends TypographyTypeBold {
  @override
  double get size => 34;
}

// -- HeadBold --

class HeadBold20px extends TypographyTypeHeadBold {
  @override
  double get size => 20;
}
